@@ -1,5 +1,5 @@
 ;;;
-;;; Copyright (c) 2003-2006 uim Project http://uim.freedesktop.org/
+;;; Copyright (c) 2003-2007 uim Project http://uim.freedesktop.org/
 ;;;
 ;;; All rights reserved.
 ;;;
@@ -181,7 +181,7 @@
       ((not (member (charcode->string key) (rk-expect rkc)))
        (rk-flush rkc)
        (im-commit-raw pc))
-      (#t	; `else' is not supported by uim's scheme interpreter
+      (else
        (set! res (rk-push-key! rkc (charcode->string key)))))
     (if res
 	(im-commit pc
@@ -232,7 +232,19 @@
 (define (tutcode-init-handler id im arg)
   (tutcode-context-new id im))
 
-(define (tutcode-reset-handler tc) #f)
+(define (tutcode-reset-handler tc)
+  (let ((rkc (tutcode-context-rk-context tc)))
+    (rk-flush rkc)))
+
+(define (tutcode-focus-in-handler tc) #f)
+
+(define (tutcode-focus-out-handler tc)
+  (let ((rkc (tutcode-context-rk-context tc)))
+    (rk-flush rkc)))
+
+(define tutcode-place-handler tutcode-focus-in-handler)
+(define tutcode-displace-handler tutcode-focus-out-handler)
+
 (define (tutcode-get-candidate-handler tc idx) #f)
 (define (tutcode-set-candidate-index-handler tc idx) #f)
 
@@ -254,7 +266,13 @@
  tutcode-reset-handler
  tutcode-get-candidate-handler
  tutcode-set-candidate-index-handler
- context-prop-activate-handler)
+ context-prop-activate-handler
+ #f
+ tutcode-focus-in-handler
+ tutcode-focus-out-handler
+ tutcode-place-handler
+ tutcode-displace-handler
+ )
 
 ;;; ơ֥QWERTYܡѡ
 (define tutcode-rule
